#!/usr/bin/env python
# -*- coding: utf-8 -*-
# dicompyler_app.py
"""Script to start dicompyler without installing from source."""
# Copyright (c) 2009-2014 Aditya Panchal
# This file is part of dicompyler, released under a BSD license.
#    See the file license.txt included with this distribution, also
#    available at http://code.google.com/p/dicompyler/

import dicompyler.main

dicompyler.main.start()
